#include <kdb.h>
#include <kdbconfig.h>
#include <kdbplugin.h>

#include <string.h>

#include <kdberrors.h>

#define STRINGIFY(x) STRINGIFY2(x)
#define STRINGIFY2(x) #x

int elektraConstantsGet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned ELEKTRA_UNUSED, Key *parentKey ELEKTRA_UNUSED);
int elektraConstantsSet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned ELEKTRA_UNUSED, Key *parentKey ELEKTRA_UNUSED);

static void elektraAddKeyValue(KeySet *ks, Key *parentKey, const char *key, const char *value)
{
	Key *k = keyDup(parentKey);
	keyAddName(k, key);
	keySetString(k, value);
	ksAppendKey(ks, k);
}

static void elektraAddConstants(KeySet *ks, Key *parentKey)
{
	Key * key = keyDup (parentKey);
	ksAppendKey(ks, key);

	key = keyDup (parentKey);
	keyAddName(key, "version");
	keySetString(key, "Duplicate of version info also found below system/elektra/version");
	ksAppendKey(ks, key);


	key = keyDup (key);
	keyAddName(key, "version/KDB_VERSION");
	keySetString(key, "0.8.12");
	ksAppendKey(ks, key);

	key = keyDup (parentKey);
	keyAddName(key, "version/KDB_VERSION_MAJOR");
	keySetString(key, "0");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "version/KDB_VERSION_MINOR");
	keySetString(key, "8");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "version/KDB_VERSION_MICRO");
	keySetString(key, "12");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "version/SO_VERSION");
	keySetString(key, "4");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake");
	keySetString(key, "All cmake variables as defined in the file cmake/ElektraCache.cmake");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/PLUGINS");
	keySetString(key, "dump;resolver;sync;error;template;doc;ccode;fstab;hexcode;hidden;ni;null;struct;tracer;counter;type;constants;noresolver;wresolver;ini;glob;hosts;iconv;network;path;keytometa;rename;syslog;uname;timeofday;simpleini;line;validation;regexstore;resolver_fm_b_b;resolver_fm_hb_b;resolver_fm_hp_b;resolver_fm_ub_x;resolver_fm_xb_x;resolver_fm_xp_x;resolver_fm_xhp_x;resolver_fm_uhb_xb;yajl;dbus;tcl;xmltool;augeas;journald;jni");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/TOOLS");
	keySetString(key, "kdb;gen;race;qt-gui");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/KDB_DB_SYSTEM");
	keySetString(key, "/etc/kdb");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/KDB_DB_HOME");
	keySetString(key, "/home");
	ksAppendKey(ks, key);

	elektraAddKeyValue(ks, parentKey, "cmake/KDB_DB_SPEC", "share/elektra/specification");
	elektraAddKeyValue(ks, parentKey, "cmake/KDB_DB_DIR", ".dir");

	key = keyDup (parentKey);
	keyAddName(key, "cmake/KDB_DB_USER");
	keySetString(key, ".config");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/ENABLE_CXX11");
	keySetString(key, "OFF");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/GTEST_ROOT");
	keySetString(key, "");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/BUILD_SHARED");
	keySetString(key, "ON");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/BUILD_FULL");
	keySetString(key, "ON");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/BUILD_STATIC");
	keySetString(key, "OFF");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/BINDINGS");
	keySetString(key, "cpp");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/ELEKTRA_DEBUG_BUILD");
	keySetString(key, "OFF");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/ELEKTRA_VERBOSE_BUILD");
	keySetString(key, "OFF");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/TARGET_INCLUDE_FOLDER");
	keySetString(key, "elektra");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/TARGET_CMAKE_FOLDER");
	keySetString(key, "/usr/lib64/cmake");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/TARGET_PLUGIN_FOLDER");
	keySetString(key, "elektra");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/TARGET_PKGCONFIG_FOLDER");
	keySetString(key, "pkgconfig");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/TARGET_TOOL_EXEC_FOLDER");
	keySetString(key, "lib64/elektra/tool_exec");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/TARGET_TEST_DATA_FOLDER");
	keySetString(key, "share/elektra/test_data");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/TARGET_TEMPLATE_FOLDER");
	keySetString(key, "share/elektra/templates");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/LIB_SUFFIX");
	keySetString(key, "64");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "cmake/CMAKE_INSTALL_PREFIX");
	keySetString(key, "/usr");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "compiler");
	keySetString(key, "Flags defined for compilers as defined in ElektraCompiling.cmake");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "compiler/c_flags");
	keySetString(key, "-O2 -g -pipe -Wall -Werror=format-security -Wp,-D_FORTIFY_SOURCE=2 -fexceptions -fstack-protector-strong --param=ssp-buffer-size=4 -grecord-gcc-switches -specs=/usr/lib/rpm/redhat/redhat-hardened-cc1 -m64 -mtune=generic  -std=gnu99  -Wno-deprecated-declarations  -Wno-long-long -pedantic -Wno-variadic-macros -Wall -Wextra -Wno-overlength-strings -Wsign-compare -Wfloat-equal -Wformat-security -Wshadow -Wcomments -Wtrigraphs -Wundef -Wuninitialized -Winit-self -Wmaybe-uninitialized -Wsign-compare -Wfloat-equal -Wformat-security");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "compiler/id");
	keySetString(key, "GNU");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "compiler/coverage");
	keySetString(key, "OFF");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "compiler/cxx_flags");
	keySetString(key, "-O2 -g -pipe -Wall -Werror=format-security -Wp,-D_FORTIFY_SOURCE=2 -fexceptions -fstack-protector-strong --param=ssp-buffer-size=4 -grecord-gcc-switches -specs=/usr/lib/rpm/redhat/redhat-hardened-cc1 -m64 -mtune=generic  -std=c++98  -Wno-deprecated-declarations  -Wstrict-null-sentinel -Wno-missing-field-initializers -Wold-style-cast -Woverloaded-virtual  -Wsign-promo  -Wno-long-long -pedantic -Wno-variadic-macros -Wall -Wextra -Wno-overlength-strings -Wsign-compare -Wfloat-equal -Wformat-security -Wshadow -Wcomments -Wtrigraphs -Wundef -Wuninitialized -Winit-self -Wmaybe-uninitialized");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "compiler/pic_flags");
	keySetString(key, "-fPIC");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "compiler/static_flags");
	keySetString(key, "");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "macros");
	keySetString(key, "Define the macros from kdbos.h and kdbconfig.h");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "macros/KDB_MAX_PATH_LENGTH");
	keySetString(key, STRINGIFY(KDB_MAX_PATH_LENGTH));
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "macros/KDB_FILE_MODE");
	keySetString(key, STRINGIFY(KDB_FILE_MODE));
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "macros/KDB_DIR_MODE");
	keySetString(key, STRINGIFY(KDB_DIR_MODE));
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "macros/KDB_PATH_SEPARATOR");
	keySetString(key, STRINGIFY(KDB_PATH_SEPARATOR));
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "macros/KDB_PATH_ESCAPE");
	keySetString(key, STRINGIFY(KDB_PATH_ESCAPE));
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package");
	keySetString(key, "Infos about the package as it is installed");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package/prefix");
	keySetString(key, "/usr");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package/libdir");
	keySetString(key, "/usr/lib64");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package/includedir");
	keySetString(key, "/usr/include/elektra");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package/plugindir");
	keySetString(key, "/usr/lib64/elektra");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package/tool_execdir");
	keySetString(key, "/usr/lib64/elektra/tool_exec");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package/templatedir");
	keySetString(key, "/usr/share/elektra/templates");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package/libs");
	keySetString(key, "-L/usr/lib64 -lelektra");
	ksAppendKey(ks, key);


	key = keyDup (parentKey);
	keyAddName(key, "package/cflags");
	keySetString(key, "-I/usr/include/elektra");
	ksAppendKey(ks, key);
}

int elektraConstantsGet(Plugin *handle ELEKTRA_UNUSED, KeySet *ks, Key *parentKey)
{
	if (!strcmp (keyName(parentKey), "system/elektra/modules/constants"))
	{
		ksAppendKey(ks, keyNew ("system/elektra/modules/constants",
			KEY_VALUE, "constants plugin waits for your orders", KEY_END));
		ksAppendKey(ks, keyNew ("system/elektra/modules/constants/exports", KEY_END));
		ksAppendKey(ks, keyNew ("system/elektra/modules/constants/exports/get",
			KEY_FUNC, elektraConstantsGet,
			KEY_END));
		ksAppendKey(ks, keyNew ("system/elektra/modules/constants/exports/set",
			KEY_FUNC, elektraConstantsSet,
			KEY_END));
		KeySet *contract = ksNew(30,
#include "readme_constants.c"
			keyNew ("system/elektra/modules/constants/infos/version",
				KEY_VALUE, PLUGINVERSION, KEY_END),
			KS_END);
		ksAppend(ks, contract);
		ksDel(contract);
		return 1;
	}

	elektraAddConstants(ks, parentKey);

	return 1; /* success */
}

int elektraConstantsSet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *parentKey)
{
	KeySet *info = ksNew(0, KS_END);
	elektraAddConstants(info, parentKey);

	Key *k;
	ksRewind(info);
	ksRewind(returned);
	while ((k = ksNext(returned)))
	{
		Key *c = ksNext(info);
		if (strcmp(keyName(k), keyName(c)) || strcmp(keyString(k), keyString(c)))
		{
			ELEKTRA_SET_ERRORF(84, parentKey, "the key %s (expected %s) was modified to %s (expected %s)", keyName(k), keyName(c), keyString(k), keyString(c));
			return -1;
		}
	}

	ksDel (info);
	return 0;
}

Plugin *ELEKTRA_PLUGIN_EXPORT(constants)
{
	return elektraPluginExport("constants",
		ELEKTRA_PLUGIN_GET,	&elektraConstantsGet,
		ELEKTRA_PLUGIN_SET,	&elektraConstantsSet,
		ELEKTRA_PLUGIN_END);
}

